from emod_api import schema_to_class as s2c
import json

def do_nodes( schema_path, node_ids ):
    """
        Create and return a NodeSetConfig based on node_ids list.
    """
    if node_ids and len(node_ids) > 0:
        nodelist = s2c.get_class_with_defaults( "NodeSetNodeList", schema_path )
        nodelist.Node_List = node_ids
    else:
        nodelist = s2c.get_class_with_defaults( "NodeSetAll", schema_path )
    return nodelist

def get_waning_from_params( schema_path, initial=1.0, box_duration=365, decay_rate=0 ):
    """
        Get well configured waning structure. Default is 1-year full efficacy box.
        Note that an infinite decay rate (0 or even -1) is same as Box.
        Note that an infinite box duration (-1) is same as constant.
        Note that a zero box duration is same as Exponential.

    """
    waning = None
    if box_duration == -1:
        waning = s2c.get_class_with_defaults( "WaningEffectConstant", schema_path )
    else:
        waning = s2c.get_class_with_defaults( "WaningEffectBoxExponential", schema_path )
        waning.Box_Duration = box_duration
        if decay_rate > 0: # -1 really
            waning.Decay_Time_Constant = 1.0/decay_rate
        else:
            waning.Decay_Time_Constant = 0

    waning.Initial_Effect = initial
    return waning


